@@ -16,13 +16,16 @@
 # Fifth Floor, Boston, MA 02110-1301 USA.
 
 require 'singleton'
+require 'observer'
+require 'alexandria/net'
 
 module Alexandria
   class BookProviders < Array
     include Logging
     include Singleton
+    include Observable
     include GetText
-    GetText.bindtextdomain(Alexandria::TEXTDOMAIN, nil, nil, "UTF-8")
+    GetText.bindtextdomain(Alexandria::TEXTDOMAIN, :charset => "UTF-8")
 
     SEARCH_BY_ISBN, SEARCH_BY_TITLE, SEARCH_BY_AUTHORS,
     SEARCH_BY_KEYWORD = (0..3).to_a
@@ -34,20 +37,35 @@ module Alexandria
 
     def self.search(criterion, type)
       factory_n = 0
+      #puts "book_providers search #{self.instance.count_observers}"
+
       begin
         factory = self.instance[factory_n]
         puts factory.fullname + " lookup" if $DEBUG
+        self.instance.changed
+        self.instance.notify_observers(:searching, factory.fullname) # new
         results = factory.search(criterion, type)
 
         if results.length == 0
+          self.instance.changed
+          self.instance.notify_observers(:not_found, factory.fullname) # new
           raise NoResultsError
         else
-          puts "found at " + factory.fullname
+          log.info { "found at " + factory.fullname }
+          self.instance.changed
+          self.instance.notify_observers(:found, factory.fullname) # new
           return results
         end
       rescue Exception => boom
+        unless boom.instance_of? NoResultsError
+          self.instance.changed
+          self.instance.notify_observers(:error, factory.fullname) # new
+          Thread.new {sleep(0.5)}.join # hrmmmm, to make readable...
+          trace = boom.backtrace.join("\n> ")
+          log.warn { "Provider #{factory.name} encountered error: #{boom.message} #{trace}" }
+        end
         if self.last == factory
-          puts "Error while searching #{criterion}"
+          log.warn { "Error while searching #{criterion}" }
           raise case boom
                 when Timeout::Error
                   _("Couldn't reach the provider '%s': timeout " +
@@ -232,37 +250,78 @@ module Alexandria
       unabstract
     end
 
-    require 'alexandria/book_providers/bn'
-    require 'alexandria/book_providers/proxis'
-    require 'alexandria/book_providers/mcu'
-    require 'alexandria/book_providers/thalia'
-    require 'alexandria/book_providers/ibs_it'
-    require 'alexandria/book_providers/renaud'
-    require 'alexandria/book_providers/adlibris'
-    require 'alexandria/book_providers/ls'
-    require 'alexandria/book_providers/bol_it'
-    require 'alexandria/book_providers/webster_it'
-    require 'alexandria/book_providers/worldcat'
+    #require 'alexandria/book_providers/bn'
+    log.info { "Not loading Barnes and Noble (provider not functional)" }
 
-    begin
-      require 'alexandria/book_providers/amazon'
-    rescue LoadError
-      log.info { "Can't load Ruby/Amazon, hence provider Amazon not available" }
-    end
+    require 'alexandria/book_providers/proxis'
+
+    require 'alexandria/book_providers/mcu' #?
+
+    #require 'alexandria/book_providers/thalia'
+    log.info { "Not loading Thalia (provider not functional)" }
+
+    #require 'alexandria/book_providers/ibs_it'
+    log.info { "Not loading IBS (provider not functional)" }
+
+    #require 'alexandria/book_providers/renaud'
+    log.info { "Not loading Renaud (provider not functional)" }
+
+
+    #require 'alexandria/book_providers/adlibris' # new, now needs other deps
+    ## require 'alexandria/book_providers/ls' # obsolete, replaced by siciliano
+    #require 'alexandria/book_providers/bol_it'
+    log.info { "Not loading BOL (provider not functional)" }
+
+    #require 'alexandria/book_providers/webster_it'
+    log.info { "Not loading Webster (provider not functional)" }
+
+    #require 'alexandria/book_providers/worldcat' # new, now needs hpricot
 
     # mechanize is optional
+    #begin
+    #  begin
+    #    require 'mechanize'
+    #  rescue LoadError
+    #    require 'rubygems'
+    #    require 'mechanize'
+    #  end
+    #  require 'alexandria/book_providers/dea_store_it'
+    #rescue LoadError
+    #  log.warn { "Can't load mechanize, hence provider Deastore not available" }
+    #end
+
+    # Amazon AWS (Amazon Associates Web Services) provider, needs hpricot
     begin
-      require 'alexandria/book_providers/dea_store_it'
+      begin
+        require 'hpricot'
+      rescue LoadError
+        require 'rubygems'
+        require 'hpricot'
+      end
+      require 'alexandria/book_providers/amazon_aws'
+      require 'alexandria/book_providers/deastore'
+      require 'alexandria/book_providers/siciliano'
+      require 'alexandria/book_providers/worldcat'
     rescue LoadError
-      log.info { "Can't load mechanize, hence provider Deastore not available" }
+      log.warn { "Can't load hpricot, hence provider Amazon not available" }
+      log.warn { "Can't load hpricot, hence provider DeaStore not available" }
+      log.warn { "Can't load hpricot, hence provider Siciliano not available" }
+      log.warn { "Can't load hpricot, hence provider WorldCat not available" }
     end
 
-    # new Amazon ECS 4 provider
+    # AdLibris (needs htmlentities and hpricot)
     begin
-      require 'alexandria/book_providers/amazon_ecs_util'
-      require 'alexandria/book_providers/amazon_ecs4'
-    rescue LoadError
-      puts "Can't load hpricot, hence provider Amazon not available"
+      begin
+        require 'htmlentities'
+        require 'hpricot'
+      rescue LoadError
+        require 'rubygems'
+        require 'htmlentities'
+        require 'hpricot'
+      end
+      require 'alexandria/book_providers/adlibris'
+    rescue LoadError => ex
+      log.warn { "Can't load provider AdLibris, requires both hpricot and htmlentities: #{ex.message}" }
     end
 
 
@@ -282,7 +341,6 @@ module Alexandria
     end
 
     def update_priority
-
       # This is weird code that sorts through the list of classes brought
       # in by requires and sorts through whether they are 'Abstract' or not,
       # adding their names to @prefs.
@@ -320,6 +378,7 @@ module Alexandria
         end
       end
       self.clear
+      rejig_providers_priority()
       priority = (@prefs.providers_priority or [])
       priority.map! { |x| x.strip }
       rest = providers.keys - priority
@@ -328,8 +387,39 @@ module Alexandria
       self.compact!
     end
 
+
+
     def self.method_missing(id, *args, &block)
       self.instance.method(id).call(*args, &block)
     end
+
+    private
+
+    def rejig_providers_priority
+      priority = (@prefs.providers_priority or [])
+      unless priority.empty?
+        changed = false
+
+        if ecs_index = priority.index("AmazonECS") 
+          priority[ecs_index] = "Amazon" # replace legacy "AmazonECS" name
+          priority.uniq! # remove any other "Amazon" from the list
+          changed = true
+        end
+        if deastore_index = priority.index("DeaStore_it")
+          priority[deastore_index] = "DeaStore"
+          changed = true
+        end
+        if worldcat_index = priority.index("Worldcat")
+          priority[worldcat_index] = "WorldCat"
+          changed = true
+        end
+        if adlibris_index = priority.index("Adlibris")
+          priority[adlibris_index] = "AdLibris"
+          changed = true
+        end
+        @prefs.providers_priority = priority if changed
+      end
+    end
+
   end
 end
