@@ -26,7 +26,7 @@ module Alexandria
     class Z3950Provider < AbstractProvider
       include Logging
       include GetText
-      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, nil, nil, "UTF-8")
+      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, :charset => "UTF-8")
 
       def initialize(name="Z3950", fullname="Z39.50")
         super
@@ -174,7 +174,7 @@ module Alexandria
       unabstract
 
       include GetText
-      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, nil, nil, "UTF-8")
+      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, :charset => "UTF-8")
 
       def initialize
         super("LOC", _("Library of Congress (Usa)"))
@@ -186,7 +186,12 @@ module Alexandria
       end
 
       def url(book)
-        "http://catalog.loc.gov/cgi-bin/Pwebrecon.cgi?DB=local&CNT=25+records+per+page&CMD=isbn+" + Library.canonicalise_isbn(book.isbn)
+        begin
+          "http://catalog.loc.gov/cgi-bin/Pwebrecon.cgi?DB=local&CNT=25+records+per+page&CMD=isbn+" + Library.canonicalise_isbn(book.isbn)
+        rescue Exception => ex
+          log.warn { "Cannot create url for book #{book}; #{ex.message}" }
+          nil
+        end
       end
     end
 
@@ -202,7 +207,7 @@ module Alexandria
       unabstract
 
       include GetText
-      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, nil, nil, "UTF-8")
+      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, :charset => "UTF-8")
 
       def initialize
         super("BL", _("British Library"))
@@ -227,7 +232,12 @@ module Alexandria
       end
 
       def url(book)
-        "http://copac.ac.uk/openurl?isbn=" + Library.canonicalise_isbn(book.isbn)
+        begin
+          "http://copac.ac.uk/openurl?isbn=" + Library.canonicalise_isbn(book.isbn)
+        rescue Exception => ex
+          log.warn { "Cannot create url for book #{book}; #{ex.message}" }
+          nil
+        end
       end
 
       #######
@@ -286,7 +296,7 @@ module Alexandria
       unabstract
 
       include GetText
-      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, nil, nil, "UTF-8")
+      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, :charset => "UTF-8")
 
       def initialize
         super("SBN", "Servizio Bibliotecario Nazionale (Italy)")
@@ -311,12 +321,17 @@ module Alexandria
       end
 
       def url(book)
-        "http://sbnonline.sbn.it/cgi-bin/zgw/BRIEF.pl?displayquery=%253CB%253E%253Cfont%2520color%253D%2523000064%253E" +
-          "Codice%2520ISBN%253C%2FB%253E%253C%2Ffont%253E%2520contiene%2520%2522%2520%253CFONT%2520COLOR%253Dred%253E" +
-          canonicalise_isbn_with_dashes(book.isbn) +
-          "%253C%2FFONT%253E%2522&session=&zurl=opac&zquery=%281%3D7+4%3D2+2%3D3+5%3D100+6%3D1+3%3D3+%22" +
-          canonicalise_isbn_with_dashes(book.isbn) +
-          "%22%29&language=it&maxentries=10&target=0&position=1"
+        begin
+          "http://sbnonline.sbn.it/cgi-bin/zgw/BRIEF.pl?displayquery=%253CB%253E%253Cfont%2520color%253D%2523000064%253E" +
+            "Codice%2520ISBN%253C%2FB%253E%253C%2Ffont%253E%2520contiene%2520%2522%2520%253CFONT%2520COLOR%253Dred%253E" +
+            canonicalise_isbn_with_dashes(book.isbn) +
+            "%253C%2FFONT%253E%2522&session=&zurl=opac&zquery=%281%3D7+4%3D2+2%3D3+5%3D100+6%3D1+3%3D3+%22" +
+            canonicalise_isbn_with_dashes(book.isbn) +
+            "%22%29&language=it&maxentries=10&target=0&position=1"
+        rescue Exception => ex
+          log.warn { "Cannot create url for book #{book}; #{ex.message}" }
+          nil
+        end
       end
 
       #######
