@@ -41,7 +41,7 @@ module Alexandria
 
     include GetText
     extend GetText
-    bindtextdomain(Alexandria::TEXTDOMAIN, nil, nil, "UTF-8")
+    bindtextdomain(Alexandria::TEXTDOMAIN, :charset => "UTF-8")
 
     BOOK_ADDED, BOOK_UPDATED, BOOK_REMOVED = (0..3).to_a
     include Observable
@@ -91,13 +91,12 @@ module Alexandria
             next
           end
           book = self.regularize_book_from_yaml(test[1])
-          book.library = library
           old_isbn = book.isbn
           old_pub_year = book.publishing_year
           begin
             begin
               book.isbn = self.canonicalise_ean(book.isbn).to_s unless book.isbn == nil
-              raise "Not a book: #{text.inspect}" unless book.is_a?(Book)
+              raise "Not a book: #{book.inspect}" unless book.is_a?(Book)
             rescue InvalidISBNError => e
               book.isbn = old_isbn
             end
@@ -107,16 +106,28 @@ module Alexandria
             # Or if isbn has changed
             raise "#{test[1]} isbn is not okay" unless book.isbn == old_isbn
 
-            # Re-save book if VERSION changes
-            raise "#{test[1]} version is not okay" unless book.version == VERSION
+            # Re-save book if Alexandria::DATA_VERSION changes
+            raise "#{test[1]} version is not okay" unless book.version == Alexandria::DATA_VERSION
 
             # Or if publishing year has changed
             raise "#{test[1]} pub year is not okay" unless book.publishing_year == old_pub_year
 
             # ruined_books << [book, book.isbn, library]
+            book.library = library.name
+
+            ## TODO copy cover image file, if necessary
+            # due to #26909 cover files for books without ISBN are re-saved as "g#{ident}.cover"
+            if book.isbn == nil || book.isbn.empty?
+              if File.exist? library.old_cover(book)
+                log.debug { "#{library.name}; book #{book.title} has no ISBN, fixing cover image" }
+                FileUtils::Verbose.mv(library.old_cover(book), library.cover(book))
+              end
+            end
+
+
             library << book
           rescue => e
-            book.version = VERSION
+            book.version = Alexandria::DATA_VERSION
             savedfilename = library.simple_save(book)
             test[0] = test[0] + 1
             test[1] = savedfilename
@@ -146,6 +157,7 @@ module Alexandria
 
 
         Dir["*" + EXT[:cover]].each do |cover|
+          next if cover[0] == 'g'
           md = /(.+)\.cover/.match(cover)
           begin
             ean = self.canonicalise_ean(md[1])
@@ -272,7 +284,7 @@ module Alexandria
     end
 
     def self.extract_numbers(isbn)
-      raise NoISBNError.new("Nil ISBN") if isbn == nil
+      raise NoISBNError.new("Nil ISBN") if isbn == nil || isbn.empty?
 
       isbn.delete('- ').upcase.split('').map do |x|
         raise InvalidISBNError.new(isbn) unless x =~ /[\dX]/
@@ -357,7 +369,10 @@ module Alexandria
         code = "978" + code[0..8]
         return code + String( self.ean_checksum( self.extract_numbers( code ) ) )
       elsif self.valid_upc?(code)
-        raise "fix function Alexandria::Library.canonicalise_ean"
+        isbn10 =  self.canonicalise_isbn
+        code = "978" + isbn10[0..8]
+        return code + String( self.ean_checksum( self.extract_numbers( code ) ) )
+        ## raise "fix function Alexandria::Library.canonicalise_ean"
       else
         raise InvalidISBNError.new(code)
       end
@@ -391,7 +406,10 @@ module Alexandria
     def simple_save(book)
       # Let's initialize the saved identifier if not already
       # (backward compatibility from 0.4.0)
-      book.saved_ident ||= book.ident
+      # book.saved_ident ||= book.ident
+      if book.saved_ident.nil? or book.saved_ident.empty?
+        book.saved_ident = book.ident()
+      end
       if book.ident != book.saved_ident
         #log.debug { "Backwards compatibility step: #{book.saved_ident.inspect}, #{book.ident.inspect}" }
         FileUtils.rm(yaml(book.saved_ident))
@@ -431,7 +449,8 @@ module Alexandria
       ##was File.exists? but that returns true for empty files... CathalMagus
       already_there = (File.size?(yaml(book)) and
                        !@deleted_books.include?(book))
-      temp_book=book
+      
+      temp_book=book.dup
       temp_book.library=nil
       File.open(yaml(temp_book), "w") { |io| io.puts temp_book.to_yaml }
 
@@ -544,10 +563,18 @@ module Alexandria
       return filtered_library
     end
 
+    def old_cover(book)
+      File.join(self.path, book.ident.to_s + EXT[:cover])
+    end
+
     def cover(something)
       ident = case something
               when Book
-                something.ident
+                if (something.isbn && (not something.isbn.empty?))
+                  something.ident
+                else
+                  "g#{something.ident}" # g is for generated id...
+                end
               when String
                 something
               when Bignum
@@ -608,6 +635,7 @@ module Alexandria
     end
 
     def final_cover(book)
+      # TODO what about PNG?
       book.ident + (Library.jpeg?(cover(book)) ? '.jpg' : '.gif')
     end
 
@@ -631,6 +659,7 @@ module Alexandria
       @all_libraries.clear
       @all_libraries.concat(Library.loadall)
       @all_libraries.concat(SmartLibrary.loadall)
+      
       ruined = []
       deleted = []
       last = []
@@ -650,6 +679,10 @@ module Alexandria
     def all_smart_libraries
       @all_libraries.select { |x| x.is_a?(SmartLibrary) }
     end
+    
+    #def all_dynamic_libraries
+    #      @all_libraries.select { |x| x.is_a?(SmartLibrary) }
+    #end
 
     LIBRARY_ADDED, LIBRARY_REMOVED = 1, 2
 
