@@ -26,7 +26,7 @@ module Alexandria
   module UI
     class KeepBadISBNDialog < AlertDialog
       include GetText
-      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, nil, nil, "UTF-8")
+      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, :charset => "UTF-8")
 
       def initialize(parent, book)
         super(parent, _("Invalid ISBN '%s'") % book.isbn,
@@ -51,7 +51,7 @@ module Alexandria
       include Logging
       include GetText
       extend GetText
-      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, nil, nil, "UTF-8")
+      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, :charset => "UTF-8")
 
       def initialize(parent, selected_library=nil, &block)
         super('new_book_dialog.glade')
@@ -125,14 +125,31 @@ module Alexandria
       end
 
       def on_criterion_toggled(item)
+
+
         log.debug { "on_criterion_toggled" }
         return unless item.active?
+
+        # There used to be a strange effect here (pre SVN r1022).
+        # When item is first toggled to "Search" the entry_search
+        # field was unselectable. One used to have to click the dialog
+        # title bar to be able to focus it again. Putting the GUI
+        # modifications in an Gtk.idle_add block fixed the problem.
+
         if is_isbn = item == @isbn_radiobutton
-          @latest_size = @new_book_dialog.size
-          @new_book_dialog.resizable = false
+          Gtk.idle_add do
+            @latest_size = @new_book_dialog.size
+            @new_book_dialog.resizable = false            
+            @entry_isbn.grab_focus
+            false
+          end
         else
-          @new_book_dialog.resizable = true
-          @new_book_dialog.resize(*@latest_size) unless @latest_size.nil?
+          Gtk.idle_add do
+            @new_book_dialog.resizable = true
+            @new_book_dialog.resize(*@latest_size) unless @latest_size.nil?
+            @entry_search.grab_focus
+            false
+          end
         end
         @entry_isbn.sensitive = is_isbn
         @combo_search.sensitive = !is_isbn
@@ -148,6 +165,9 @@ module Alexandria
         # Remember the last criterion selected (so that we can re-select
         # it when the dialog opens again).
         @@last_criterion_was_not_isbn = !is_isbn
+
+        # @new_book_dialog.present # attempted fix, bring dialog to foreground
+
       end
 
       def on_changed(entry)
@@ -234,15 +254,15 @@ module Alexandria
                  BookProviders::SEARCH_BY_KEYWORD
                end
 
-        @progressbar.show 
-        progress_pulsing = Gtk.timeout_add(100) do
-          unless @destroyed
-            @progressbar.pulse
-            true
-          else
-            false
-          end
-        end
+        # @progressbar.show 
+        #progress_pulsing = Gtk.timeout_add(100) do
+        #  unless @destroyed
+        #    @progressbar.pulse
+        #    true
+        #  else
+        #    false
+        #  end
+        #end
 
         criterion = @entry_search.text.strip
         @treeview_results.model.clear
@@ -255,13 +275,19 @@ module Alexandria
         @find_thread.kill if @find_thread
         @image_thread.kill if @image_thread
 
+        notify_start_add_by_isbn
         @find_thread = Thread.new do
           log.info { "New @find_thread #{Thread.current}" }
           begin
+            Alexandria::BookProviders.instance.add_observer(self)
             @results = Alexandria::BookProviders.search(criterion, mode)
+            
             log.info { "got #{@results.length} results" }
           rescue => e
             @find_error = e.message
+          ensure
+            Alexandria::BookProviders.instance.delete_observer(self)
+            #   notify_end_add_by_isbn
           end
         end
 
@@ -313,8 +339,9 @@ module Alexandria
           unless continue
             unless @find_thread.alive? #This happens after find_thread is done
               unless @destroyed
-                Gtk.timeout_remove(progress_pulsing)
-                @progressbar.hide
+                # Gtk.timeout_remove(progress_pulsing)
+                # @progressbar.hide
+                notify_end_add_by_isbn
                 @button_add.sensitive = false
               end
             end
@@ -324,7 +351,7 @@ module Alexandria
         end
       end
 
-      def decode_cuecat?(entry)
+      def decode_cuecat?(entry) # srsly? 
         if entry.text=~/^\..*?\..*?\.(.*?)\.$/
           tmp = $1.tr('a-zA-Z0-9+-', ' -_')
           tmp = ((32 + tmp.length * 3/4).to_i.chr << tmp).unpack('u')[0]
@@ -343,6 +370,118 @@ module Alexandria
 
           on_add
         end
+      end
+
+      def add_single_book_by_isbn(library, is_new)
+        # Perform the ISBN search via the providers.
+        isbn = begin
+                 Library.canonicalise_isbn(@entry_isbn.text)
+               rescue
+                 raise _("Couldn't validate the EAN/ISBN you " +
+                         "provided.  Make sure it is written " +
+                         "correctly, and try again.")
+               end
+        assert_not_exist(library, @entry_isbn.text)        
+        @button_add.sensitive = false  
+        notify_start_add_by_isbn
+        @find_thread = Thread.new do
+          log.info { "New @find_thread #{Thread.current}" }
+          begin
+            # MAJOR HACK, add this again...
+            Alexandria::BookProviders.instance.add_observer(self)
+            book, cover_url = Alexandria::BookProviders.isbn_search(isbn)
+            # prov = FakeBookProviders.new()
+            # prov.add_observer(self)
+            # book, cover_url = prov.isbn_search(isbn)
+
+            notify_end_add_by_isbn
+
+            if book
+
+              puts "adding book #{book} to library"
+              add_book_to_library(library, book, cover_url)
+              @entry_isbn.text = ''
+              
+              post_addition([book], library, is_new)
+            else
+              post_addition([], library, is_new)
+            end
+          rescue => e
+            puts e.message
+            puts e.backtrace.join("\n> ")
+            @find_error = e.message
+            @button_add.sensitive = true
+            notify_end_add_by_isbn
+          ensure
+            Alexandria::BookProviders.instance.delete_observer(self)
+          end          
+        end
+      end
+
+      def add_selected_books(library, is_new)
+        books_to_add = []
+        @treeview_results.selection.selected_each do |model, path, iter|
+          @results.each do |book, cover|
+            next unless book.ident == iter[1]
+            begin
+              next unless assert_not_exist(library, book.isbn)
+            rescue Alexandria::Library::NoISBNError
+              puts "noisbn"
+              book.isbn = book.saved_ident = nil
+              books_to_add << [book, cover]
+              next
+            rescue Alexandria::Library::InvalidISBNError
+              puts "invalidisbn #{book.isbn}"
+              next unless
+                KeepBadISBNDialog.new(@parent, book).keep?
+              book.isbn = book.saved_ident = nil
+            end
+            books_to_add << [book, cover]
+          end
+          
+        end
+        books_to_add.each do |book, cover_uri|
+          add_book_to_library(library, book, cover_uri)
+        end
+        return books_to_add.map {|x| x.first } # array of Books only
+      end
+
+      def add_book_to_library(library, book, cover_uri)
+        unless cover_uri.nil?
+          library.save_cover(book, cover_uri)
+        end
+        library << book
+        library.save(book)
+      end
+
+      def post_addition(books, library, is_new_library)
+        puts "post_addition #{books.size}" 
+        return if books.empty?
+
+        # books, a 1d array of Alexandria::Book        
+        @block.call(books, library, is_new_library)
+
+        if @keep_open.active?
+          # TODO reset and clear fields
+          if @@last_criterion_was_not_isbn
+            @entry_search.select_region(0, -1) # select all, ready to delete
+            @treeview_results.model.clear
+            @entry_search.grab_focus
+          else
+            @button_add.sensitive = true #
+            @entry_isbn.text = '' # blank ISBN field
+            @entry_isbn.grab_focus
+          end
+          
+        else
+          # Now we can destroy the dialog and go back to the main
+          # application.
+          @new_book_dialog.destroy
+        end
+        
+        
+        
+
       end
 
       def on_add
@@ -352,65 +491,26 @@ module Alexandria
 
         begin
           libraries = Libraries.instance.all_libraries
-          library, new_library =
+          library, is_new_library =
             @combo_libraries.selection_from_libraries(libraries)
-          books_to_add = []
+
+          book_was_added = false
+
           if @isbn_radiobutton.active?
-            # Perform the ISBN search via the providers.
-            isbn = begin
-                     Library.canonicalise_isbn(@entry_isbn.text)
-                   rescue
-                     raise _("Couldn't validate the EAN/ISBN you " +
-                             "provided.  Make sure it is written " +
-                             "correctly, and try again.")
-                   end
-            assert_not_exist(library, @entry_isbn.text)
-            books_to_add << Alexandria::BookProviders.isbn_search(isbn)
+            add_single_book_by_isbn(library, is_new_library)
           else
-            @treeview_results.selection.selected_each do |model, path,
-              iter|
-            @results.each do |book, cover|
-              next unless book.ident == iter[1]
-              begin
-                next unless
-                assert_not_exist(library, book.isbn)
-              rescue Alexandria::Library::InvalidISBNError
-                next unless
-                KeepBadISBNDialog.new(@parent, book).keep?
-                book.isbn = book.saved_ident = nil
-              rescue Alexandria::Library::NoISBNError
-                book.isbn = book.saved_ident = nil
-                books_to_add << [book, cover]
-                next
-              end
-              books_to_add << [book, cover]
-
-            end
-            end
-          end
-
-          # Save the books in the library.
-          books_to_add.each do |book, cover_uri|
-            unless cover_uri.nil?
-              library.save_cover(book, cover_uri)
-            end
-            library << book
-            library.save(book)
+            books = add_selected_books(library, is_new_library)
+            post_addition(books, library, is_new_library)
           end
 
           # Do not destroy if there is no addition.
-          return if books_to_add.empty?
+          #          return unless book_was_added
 
-          # Now we can destroy the dialog and go back to the main
-          # application.
-          @new_book_dialog.destroy
-          @block.call(books_to_add.map { |x| x.first },
-                      library,
-                      new_library)
+          
         rescue => e
           ErrorDialog.new(@parent, _("Couldn't add the book"), e.message)
         end
-        books_to_add
+        #books_to_add
       end
 
       def on_cancel
@@ -419,13 +519,55 @@ module Alexandria
         @new_book_dialog.destroy
       end
 
+      def notify_start_add_by_isbn
+        main_progress_bar = MainApp.instance.appbar.children.first
+        main_progress_bar.visible = true
+        @progress_pulsing = Gtk.timeout_add(100) do
+          unless @destroyed
+            main_progress_bar.pulse
+            true
+          else
+            false
+          end
+        end
+      end
+
+      def notify_end_add_by_isbn
+        MainApp.instance.appbar.children.first.visible = false
+        Gtk::timeout_remove(@progress_pulsing)
+      end
+
+      def update(status, provider)
+        Gtk.idle_add do
+          messages = {
+            :searching => _("Searching Provider '%s'..."),
+            :error => _("Error while Searching Provider '%s'"),
+            :not_found => _("Not Found at Provider '%s'"),
+            :found => _("Found at Provider '%s'")
+          }
+          message = messages[status] % provider
+          log.debug { "update message : #{message}" }
+          # @parent.appbar.status = message
+          MainApp.instance.appbar.status = message # HACKish
+          false
+        end
+      end
+
       def on_focus
         if @isbn_radiobutton.active? and @entry_isbn.text.strip.empty?
           clipboard = Gtk::Clipboard.get(Gdk::Selection::CLIPBOARD)
           if text = clipboard.wait_for_text
-            @entry_isbn.text = text if
-            Library.valid_isbn?(text) or Library.valid_ean?(text) or
-            Library.valid_upc?(text)
+            if Library.valid_isbn?(text) or Library.valid_ean?(text) or
+                Library.valid_upc?(text)
+              Gtk.idle_add do
+                #puts "idly"
+                @entry_isbn.text = text
+                @button_add.grab_focus 
+                false
+              end
+              log.debug { "Setting ISBN field to #{text}" }
+              puts text # required, strangely, to prevent GUI strangeness
+             end
           end
         end
       end
@@ -464,8 +606,9 @@ module Alexandria
 
       def assert_not_exist(library, isbn)
         # Check that the book doesn't already exist in the library.
-        canonical = Library.canonicalise_isbn(isbn)
-        if book = library.find { |book| book.isbn == canonical }
+        isbn13 = Library.canonicalise_ean(isbn)
+        puts isbn13
+        if book = library.find { |book| book.isbn == isbn13 }
           raise DuplicateBookException, _("'%s' already exists in '%s' (titled '%s').") % \
             [ isbn, library.name, book.title.sub("&", "&amp;") ]
         end
@@ -474,3 +617,4 @@ module Alexandria
     end
   end
 end
+
