@@ -24,7 +24,7 @@ module Alexandria
         :iconview_model, :filtered_model, :on_books_selection_changed
       include Logging
       include GetText
-      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, nil, nil, "UTF-8")
+      GetText.bindtextdomain(Alexandria::TEXTDOMAIN, :charset => "UTF-8")
 
       module Columns
         COVER_LIST, COVER_ICON, TITLE, TITLE_REDUCED, AUTHORS,
@@ -56,7 +56,7 @@ module Alexandria
         setup_books_iconview_sorting
         on_books_selection_changed
         restore_preferences
-        log.info { "At the end of it all: #{@iconview.model.inspect}" }
+        log.debug { "UI Manager initialized: #{@iconview.model.inspect}" }
       end
 
       def create_uimanager
@@ -306,6 +306,7 @@ module Alexandria
 
         if event_is_right_click event
           log.debug { "library right click!" }
+          library_already_selected = true
           if path = widget.get_path_at_pos(event.x, event.y)
             obj, path = widget.is_a?(Gtk::TreeView) \
               ? [widget.selection, path.first] : [widget, path]
@@ -313,6 +314,7 @@ module Alexandria
 
             unless obj.path_is_selected?(path)
               log.debug { "Select #{path}" }
+              library_already_selected = false
               widget.unselect_all
               obj.select_path(path)
             end
@@ -321,10 +323,25 @@ module Alexandria
           end
 
           menu = determine_library_popup widget, event
-          Gtk.idle_add do
-            menu.popup(nil, nil, event.button, event.time)
-            false
+
+          # Fixes part of bug #25021.
+          #
+          # If the library was not selected when it was right-clicked
+          # we should select the library first (we call on_focus
+          # manually, since the above call to obj.select_path(path) doesn't
+          # seem to suffice).
+          #
+          # Then we wait a while and only *then* pop up the menu.
+
+          
+          if library_already_selected
+            sensitize_library selected_library
+            Gtk.idle_add do
+              menu.popup(nil, nil, event.button, event.time)
+              false
+            end
           end
+          
         end
       end
 
@@ -436,13 +453,19 @@ module Alexandria
 
           # Sensitize providers URL
           if books.length == 1
-            all_url = false
+            b = books.first
+            no_urls = true
             BookProviders.each do |provider|
-              has_url = books.first.isbn and provider.url(books.first) != nil
-              @actiongroup[provider.action_name].sensitive = has_url
-              all_url = true if has_url and !all_url
+              has_no_url = true
+              begin
+                has_no_url = (b.isbn.nil? or b.isbn.strip.empty? or provider.url(b).nil?)
+              rescue Exception => ex
+                log.warn { "Error determining URL from #{provider.name}; #{ex.message}" }
+              end
+              @actiongroup[provider.action_name].sensitive = (not has_no_url)
+              no_urls = false unless has_no_url
             end
-            unless all_url
+            if no_urls
               @actiongroup["OnlineInformation"].sensitive = false
             end
           end
@@ -489,17 +512,25 @@ module Alexandria
             next
           end
           path = iter.path
+          unless view.model
+            next
+          end
           path = view.model.convert_path_to_child_path(path)
           path = @filtered_model.convert_path_to_child_path(path)
-          log.info { "Path for #{book.ident} is #{path}" }
+          log.debug { "Path for #{book.ident} is #{path}" }
           selection = view.respond_to?(:selection) ? @listview.selection : @iconview
           selection.unselect_all
           selection.select_path(path)
         end
-        log.info { "select_a_book: listview" }
-        select_this_book.call(book, @listview)
-        log.info { "select_a_book: listview" }
-        select_this_book.call(book, @iconview)
+        begin
+          log.debug { "select_a_book: listview" }
+          select_this_book.call(book, @listview)
+          log.debug { "select_a_book: listview" }
+          select_this_book.call(book, @iconview)
+        rescue Exception => ex
+          trace = ex.backtrace.join("\n> ")
+          log.error { "Failed to automatically select book: #{ex.message} #{trace}" }
+        end
         # TODO: Figure out why this frequently selects the wrong book!
       end
 
@@ -545,6 +576,10 @@ module Alexandria
       #######
 
       def open_web_browser(url)
+        if url.nil?
+          log.warn("Attempt to open browser with nil url")
+          return
+        end
         unless (cmd = Preferences.instance.www_browser).nil?
           Thread.new { system(cmd % "\"" + url + "\"") }
         else
@@ -645,7 +680,7 @@ module Alexandria
                     begin
                       File.delete(filename)
                     rescue Exception => ex
-                      log.warn { "Could not delete empty file #{filename}" }
+                      log.error { "Could not delete empty file #{filename}" }
                     end
                   end
 
@@ -734,11 +769,10 @@ module Alexandria
           icon = icon.tag(Icons::FAVORITE_TAG)
         end
         iter[Columns::COVER_ICON] = icon
-        log.info { "Full iter: " + (0..15).collect {|num| iter[num].inspect }.join(", ") }
+        log.debug { "Full iter: " + (0..15).collect {|num| iter[num].inspect }.join(", ") }
       end
 
       def append_book(book, tail=nil)
-        log.debug { "append #{book.title}" }
         log.debug { @model.inspect }
         iter = @model.append
         log.debug { "iter == #{iter}" }
@@ -800,10 +834,9 @@ module Alexandria
         return iter
       end
 
-      BADGE_MARKUP = "<span weight=\"heavy\" foreground=\"white\">%d</span>"
-
       def refresh_books
         log.debug { "refresh_books" }
+		@library_listview.set_sensitive(false)
         library = selected_library
         @model.clear
         @iconview.freeze
@@ -811,21 +844,28 @@ module Alexandria
         @appbar.children.first.visible = true   # show the progress bar
         @appbar.status = _("Loading '%s'...") % library.name
         total = library.length
-        n = 0
+        log.debug { "library #{library.name} length #{library.length}" }
+        n = 0       
+        
         Gtk.idle_add do
+
+          block_return = true
           book = library[n]
           if book
-            log.debug { "Running block at #{Time.now.strftime("%H:%M:%S")}" }
-            tail = append_book(book)
-            # convert to percents
+            begin
+              tail = append_book(book)
+            rescue Exception => ex
+              trace = ex.backtrace.join("\n > ")
+              log.error { "append_books failed #{ex.message} #{trace}" }
+            end
+              # convert to percents
             coeff = total / 100.0
             percent = n / coeff
             fraction = percent / 100
             log.debug { "#index #{n} percent #{percent} fraction #{fraction}" }
-            #puts "======================================================"
             @appbar.progress_percentage = fraction
             n+= 1
-            true
+              
           else
             @iconview.unfreeze
             @filtered_model.refilter
@@ -835,8 +875,11 @@ module Alexandria
             @appbar.children.first.visible = false
             # Refresh the status bar.
             on_books_selection_changed
-            false
+            @library_listview.set_sensitive(true)
+            block_return = false
           end
+          
+          block_return
         end
       end
 
@@ -890,12 +933,27 @@ module Alexandria
         library = selected_library
         view = page == 0 ? @iconview : @listview
         selection = page == 0 ? @iconview : @listview.selection
-        selection.selected_each do |treeview, path|
+
+        selection.selected_each do |the_view, path|
+          # don't use the_view which is passed in by this block
+          # as it doesn't consider the filtering for some reason
+          # see bug #24568
+          unless view.model
+            next
+          end
           path = view.model.convert_path_to_child_path(path)
           if path
             path = @filtered_model.convert_path_to_child_path(path)
-            iter = @model.get_iter(path)
-            a << book_from_iter(library, iter)
+            # FIX this sometimes returns a nil path for iconview...
+            if path
+              iter = @model.get_iter(path)
+              if iter
+                a << book_from_iter(library, iter)
+              end
+            end
+            # This used to cause a crash when manually adding the first
+            # book to a Library displayed in an Iconview
+            # TODO find root cause of this
           end
         end
         a
@@ -931,7 +989,8 @@ module Alexandria
         @actiongroup["AddBook"].sensitive = !smart
         @actiongroup["AddBookManual"].sensitive = !smart
         @actiongroup["Properties"].sensitive = true
-        @actiongroup["Delete"].sensitive = true #(@libraries.all_regular_libraries.length > 1)
+        can_delete = smart || (@libraries.all_regular_libraries.length > 1)
+        @actiongroup["Delete"].sensitive = can_delete ## true #(@libraries.all_regular_libraries.length > 1)
         log.debug { "sensitize_library delete: #{@actiongroup["Delete"].sensitive?}" }
       end
 
@@ -1053,7 +1112,7 @@ module Alexandria
       # Gets the sort order of the current library, for use by export
       def library_sort_order
         # added by Cathal Mc Ginley, 23 Oct 2007
-        log.info { "library_sort_order #{@notebook.page}: #{@iconview.model.inspect} #{@listview.model.inspect}" }
+        log.debug { "library_sort_order #{@notebook.page}: #{@iconview.model.inspect} #{@listview.model.inspect}" }
         sorted_on = current_view.model.sort_column_id
         if sorted_on
           sort_column = sorted_on[0]
@@ -1090,10 +1149,12 @@ module Alexandria
 
       def remove_library_iter
         old_iter = @library_listview.selection.selected
-        next_iter = @library_listview.selection.selected
-        next_iter.next!
+        # commenting out this code seems to fix #20681
+        # "crashes when switching to smart library mid-load"
+        #next_iter = @library_listview.selection.selected
+        #next_iter.next!
         @library_listview.model.remove(old_iter)
-        @library_listview.selection.select_iter(next_iter)
+        # @library_listview.selection.select_iter(next_iter)
       end
 
       def undoable_delete(library, books=nil)
