@@ -69,6 +69,8 @@ class AlexandriaBuild < Rake::TaskLib
   attr_accessor :name, :version, :rubyforge_name
   attr_accessor :author, :email, :summary, :description
 
+  attr_accessor :display_version, :data_version
+
   # config objects
   attr_accessor :files
   attr_accessor :rdoc
@@ -249,13 +251,13 @@ class AlexandriaBuild < Rake::TaskLib
 
   def define_package_tasks
     namespace 'pkg' do
-      Rake::PackageTask.new(@name, @version) do |p|
+      Rake::PackageTask.new(@name, display_version) do |p| # HACK
         p.need_tar_gz = true
         p.package_files.include("README*", "COPYING", "ChangeLog", "INSTALL",
                                 "NEWS",
                                 "Rakefile", "TODO", "alexandria.desktop",
                                 "alexandria.desktop.in", "tasks.rb",
-                                "bin/**/*", "data/**/*",
+                                "bin/**/*", "data/**/*", "misc/**/*",
                                 "doc/**/*", "lib/**/*", "po/**/*",
                                 "schemas/**/*", "spec/**/*", "tests/**/*")
       end
@@ -283,6 +285,10 @@ class AlexandriaBuild < Rake::TaskLib
 
     task :install_files do
       @install.groups.each do |src, files, dest, mode|
+        unless files.respond_to? :each
+          # String#each removed in ruby 1.9 (c.f. bug #25678)
+          files =  FileList[files]
+        end
         files.each do |file|
           install_file(src, file, dest, mode)
         end
@@ -330,6 +336,7 @@ class AlexandriaBuild < Rake::TaskLib
       default_groups.push(*manpage_installation)
       default_groups.push(*menu_installation)
       default_groups.push(*documentation_installation)
+      default_groups.push(*sounds_installation)
       default_groups
     end
 
@@ -348,14 +355,14 @@ class AlexandriaBuild < Rake::TaskLib
         filename =~ /.*\/(.+)\/.+/
           size = $1
         dest = File.join(icon_dir, size, 'apps')
-        icon_group << [File.dirname(filename), filename, dest, 0644]
+        icon_group << [File.dirname(filename), FileList[filename], dest, 0644]
       end
       icon_group
     end
 
     def desktop_installation
       desktop_dir = File.join(sharedir, 'applications')
-      [['.', build.files.desktop, desktop_dir, 0644]]
+      [['.', FileList[build.files.desktop], desktop_dir, 0644]]
     end
 
     def locale_installation
@@ -381,7 +388,16 @@ class AlexandriaBuild < Rake::TaskLib
 
     def menu_installation
       menu_dir = File.join(sharedir, 'menu')
-      [['data/menu', build.files.menu, menu_dir, 0644]]
+      [['data/menu', FileList[build.files.menu], menu_dir, 0644]]
+    end
+
+    def sounds_installation
+      sound_files = []
+      Dir.glob('data/sounds/**/*.wav') do |f|
+        sound_files << f       
+      end
+      sound_dir = File.join(sharedir, 'sounds')
+      [['data/sounds', sound_files, sound_dir, 0644]]
     end
 
     def manpage_installation
